;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;               PROOF CHECKER FOR DURATION CALCULUS IN PVS
;;; File: dc-doc.lisp
;;; Date: Sun Oct 12 1997
;;; By:
;;;   Soeren T. Heilmann, sh@it.dtu.dk
;;;   Dept. of Information Technology, Technical University of Denmark
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package 'pvs)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; DOCUMENTING DC STRATEGIES

(defun doc-step (rule-name)
  (with-output-to-string 
   (*standard-output*)
   (funcall (help-rule-fun rule-name) nil))
  )
  
(defun split-line (s)
  (let ((nlpos (position #\Newline s)))
    (if nlpos
	(cons (subseq s 0 nlpos)
	      (subseq s (+ nlpos 1)))
      (cons s ""))))

(defun empty-line (s)
  (equal "" (string-left-trim '(#\Space #\Tab) s)))


(defun split-paragraph (s)
  (let* ((split (split-line s))
	 (first (car split))
	 (s (cdr split)))
    (if (empty-line first)
	(cons "" s)
      (let* ((split (split-paragraph s))
	     (rest (car split))
	     (s (cdr split)))
	(cons (concatenate 'string first (format nil "~%") rest) s)))))

(defun doc-dc-step (name short)
  (let* ((nm (string name))
	 (lownm (string-downcase nm))
	 (lowshort (string-downcase short))
         (head (format nil "{\\LARGE \\texttt{~a:}}~a" lownm short))
         (tocentry (format nil "\\texttt{~a:}~a" lownm short))
	 (doc (cdr (split-line (doc-step name))))
	 (split (split-paragraph doc))
	 (sig (remove #\: (remove #\/ (remove #\$ (string-downcase (car split))))))
	 (split (split-paragraph (cdr split)))
	 (a (car split))
	 (e (string-right-trim '(#\Space #\Newline #\Tab) (cdr split)))
	 (args (if (equal e "") nil a))
	 (effect (if (equal e "") a e)))
    (format t "%%%~%")
    (format t "%%% ~a~%" nm)
    (format t "%%%~%")
    (format t "\\subsection*{~a}~%" head)
    (format t "\\addcontentsline{toc}{subsection}{~a}~%" tocentry)
    (format t "\\begin{description}~%")
    (format t "\\item[syntax:]~%")
    (format t "\\begin{verbatim}~%")
    (format t "~a" sig)
    (format t "\\end{verbatim}~%")
    (when args
	  (format t "\\item[arguments:]~%")
	  (format t "\\begin{verbatim}~%~%")
	  (format t "~a" args)
	  (format t "\\end{verbatim}~%"))
    (format t "\\item[effect:]~%")
    (format t "\\begin{verbatim}~%")
    (format t "~a" effect)
    (format t "\\end{verbatim}~%")
    (format t "\\end{description}~%~%")))

(defun doc-dc-group (dcsteps group heading)
  (format t "%%%%%%~%")
  (format t "%%%%%% ~a~%" heading)
  (format t "%%%%%%~%")
  (format t "\\section{~a}~%~%" heading)
  (format t "\\begin{tabular}{|l|l|}~%")
  (format t "\\hline~%")
  (format t "\\emph{Command} & \\emph{Function}\\\\\\hline~%")
  (mapcar #'(lambda (namegrpshort) 
	       (when (eq (cadr namegrpshort) group)
		     (format t "\\texttt{~a} & ~a \\\\~%" 
			     (string-downcase (string (car namegrpshort)))
			     (cddr namegrpshort))))
	   dcsteps)
  (format t "\\hline~%")
  (format t "\\end{tabular}~%~%")
  (mapcar #'(lambda (namegrpshort) 
	       (when (eq (cadr namegrpshort) group)
		     (doc-dc-step (car namegrpshort) (cddr namegrpshort))))
	   dcsteps)
  (format t "~%~%"))

(defun doc-dc ()
  (dolist (grp *dcstep-groups*) 
	  (doc-dc-group (reverse *dcsteps*) (car grp) (cdr grp))))

(defun doc-dc (filename)
  (with-open-file 
   (*standard-output* filename :direction :output :if-exists :overwrite)
   (dolist (grp *dcstep-groups*) 
	   (doc-dc-group (reverse *dcsteps*) (car grp) (cdr grp)))))

;;(doc-dc "/tmp/dc-steps.tex")
;;(doc-dc "/home/sh/pcdc_dev/dev8/dc/doc/dc-steps.tex")