(***************************************************************************
 File "autoprint.sml" Updated  1993-02-10
 Duration Calculus checker, version 2
 Functions for printing automata
 Sestoft and Skakkebaek, Computer Science, Technical University of Denmark
***************************************************************************)

signature AUTOPRINT =
sig
    eqtype natset
    eqtype 'a natfun
    eqtype 'a Bdd

    val nfaprint: (natset Bdd natfun * natset * natset * int) * string list
                  -> string
    val dfaprint: (int Bdd natfun * int * natset * int) * string list
                  -> string
end

functor Autoprint (structure Basic: BASIC 
		   and Prt: PRINTER
		   and Natsetset: NATSETSET
		   and Natfun: NATFUN
		   and Bdd: BDD) : AUTOPRINT =
struct
    open Basic Prt Natsetset Natfun Bdd

    val nl = "\n"
    val bar = "------------------------------------------------------------"
              ^ nl

    fun prtnatset ns = 
	"{" ^ prtseq (prtint, ",") (sortint (members ns)) ^ "}"

    fun prtfromto f (i,j) =
	if i>j then ""
	else f i ^ nl ^ prtfromto f (i+1, j)

    fun prtdbdd (tra, snames) i =
	let val bddi = bdd2assoc (apply tra i, snames)
	in prtint i ^ ": " ^ prtlist (prt2tup (prtstring,prtint)) bddi end
	    
    fun dfaprint((tra, ini, fin, size), snames) =
          bar
	^ "Number of states: " ^ prtint size ^ nl
	^ "Initial state:    " ^ prtint ini ^ nl
	^ "Final states:     " ^ prtnatset fin ^ nl
        ^ "Transition relation:" ^ nl 
        ^ prtfromto (prtdbdd (tra, snames)) (0,size-1) 
	^ bar

    fun prtnbdd (tra, snames) i =
	let val bddi = bdd2assoc (apply tra i, snames)
	in prtint i ^ ": " ^ prtlist (prt2tup (prtstring,prtnatset)) bddi end

    fun nfaprint((tra, ini, fin, size), snames) =
	  bar
	^ "Number of states: " ^ prtint size ^ nl
	^ "Initial states:   " ^ prtnatset ini ^ nl
	^ "Final states:     " ^ prtnatset fin ^ nl
        ^ "Transition relation:" ^ nl 
	^ prtfromto (prtnbdd (tra, snames)) (0,size-1) 
	^ bar
end
