(***************************************************************************
 File "basic.sml" Updated 1993-02-09, 1994-04-07
 Duration Calculus checker, version 2
 Auxiliary functions, sorting, and timing
 Sestoft and Skakkebaek, Computer Science, Technical University of Denmark
***************************************************************************)

signature BASIC =
    sig
	datatype 'a option = None | Some of 'a
	exception Assoc and Index
	val assoc: ''a * (''a * 'b) list -> 'b
	val index: ''a list * int -> ''a -> int
	val foldright: ('a * 'b -> 'b) -> 'a list * 'b -> 'b
	val foldleft:  ('b * 'a -> 'b) -> 'b * 'a list -> 'b
	val filter: ('a -> bool) -> 'a list -> 'a list
	val mem: ''a -> ''a list -> bool
	val rep: int -> 'a -> 'a list
	val sortstr: string list -> string list
	val sortint: int list -> int list
	val setof: ''a list -> ''a list
	val timeit: ('a -> 'b) -> 'a -> 'b * string
    end

structure Basic: BASIC =
struct
    datatype 'a option = None | Some of 'a

    exception Assoc and Index

    fun assoc (_, []) = raise Assoc
      | assoc (k, (k1,v1)::r) = if k=k1 then v1 else assoc(k, r)
	    
    fun index ([], i)    k = raise Index
      | index (k1::r, i) k = if k=k1 then i else index (r, i+1) k

    fun foldright f ([],    e) = e	       
      | foldright f (x::xs, e) = f(x, foldright f (xs,e))
	
    fun foldleft f (e, [])    = e
      | foldleft f (e, x::xs) = foldleft f (f(e,x), xs)
	
    fun filter pred [] = []
      | filter pred (x::xs) =
	if pred(x) then x :: filter pred xs  
	else  filter pred xs
	    
    fun mem x []     = false
      | mem x (y::l) = (x=y) orelse (mem x l)

    fun rep n x = if n > 0 then x :: rep (n-1) x else []
	
    (* setof removes duplicates from a list *)

    fun setof(s) =
	let fun h ([], res) = res
	      | h (x::xr, res) = 
		if mem x res then h(xr, res) else h(xr, x::res)
	in h(s, []) end
 
    fun sort leq xs =
	let fun merge([],ys) = ys 
	      | merge(xs,[]) = xs
	      | merge(x::xs, y::ys) =
		if leq(x,y) then x::merge(xs,  y::ys)
		else y::merge(x::xs, ys)
	    fun mergepairs([l], k) = [l]
	      | mergepairs(l1::l2::ls, k) =
		if k mod 2 = 1 then l1::l2::ls
		else mergepairs(merge(l1,l2)::ls, k div 2)
	    fun sorting([], ls, r)    = hd(mergepairs(ls,0))
	      | sorting(x::xs, ls, r) =
		sorting(xs, mergepairs([x]::ls, r+1), r+1)
	in case xs of 
	    [] => []
	  | xs => sorting (xs, [], 0)
	end

    fun strleq (x, y:string) = x <= y

    fun sortstr xs = sort strleq xs

    fun sortint xs = sort (op< : int*int -> bool) xs

    (** Timing in SML/NJ **** Probably due to Andrew Appel ****************)
    local type time = System.Timer.time
	val timeofday : unit -> time = 
	    System.Unsafe.CInterface.c_function "timeofday"
    in 
	fun timeit f x = 
	    let open System.Timer
		val t = start_timer()
		val rt = timeofday()
		val z = f x
		val rt' = sub_time(timeofday(),rt)
		val t' = check_timer t
		val ts = check_timer_sys t
		val tg = check_timer_gc t
	    in (z,implode["real time: ",makestring rt',
			  ", non-gc user time: ",makestring t', 
			  ", gc time: ", makestring tg, 
			  ", system time: ",makestring ts,"\n"])
	    end
    end
end (* Basic *)
