(***************************************************************************
 File "fullfischer.sml" Updated 1993-02-10
 Duration Calculus checker, version 2. 
 The full and simple Fischer algorithms specified in the diagram language
 92-12-06 Jens Ulrik Skakkebaek
 Sestoft and Skakkebaek, Computer Science, Technical University of Denmark
***************************************************************************)

(* Requires rdcdiag.sml *)

(*
     local   x: 0..2 where x=0
           d,D: integer     

               o                               o
               |                               |                               
               |                               |                               
               V                               V                               
	     -----		             -----              	  
	    /     \		            /     \             	  
     ------>|  1  |		     ------>|  6  |             	  
     |      \     /		     |      \     /             	  
     |       -----		     |       -----              	  
     |         |		     |         |                	  
     |	       |		     |         |                	  
     |         | x=0?		     |         | x=0?           	  
     |	       |		     |         |                	  
     |	       V		     |         V                	  
     |       -----		     |       -----              	  
     |      /     \		     |      /     \             	  
     |	    |  2  |		     |      |  7  |             	  
     |      \     /		     |      \     /             	  
     |       -----		     |       -----              	  
     |         |		     |         |                	  
     |	       |		     |         |                	  
     |	 [0,D] | x:=1		     |   [0,D] | x:=2           	  
     |	       |		     |         |                	  
     |	       V	    ||	     |         V                	  
     |       -----	    ||	     |       -----              	  
     |      /     \	    ||	     |      /     \             	  
     |	    |  3  |		     |      |  8  |             	  
     |      \     /		     |      \     /             	  
     |       -----		     |       -----              	  
     |         |		     |         |                	  
     |	       |		     |         |                	  
     |   [d,w] |		     |   [d,w] |                	  
     |	       |		     |         |                	  
     |	       V		     |         V                	  
     |       -----		     |       -----              	  
     | [0,w]/     \		     | [0,w]/     \             	  
     |------|  4  |		     |------|  9  |             	  
     | x<>1?\     /		     | x<>2?\     /             	  
     |       -----		     |       -----              	  
     |         |		     |         |                	  
     | 	       |		     |         |                	  
     |	  [0,w]| x=1?		     |    [0,w]| x=2?           	  
     |	       |		     |         |                	  
     |	       V		     |         V                	  
     |       -----		     |       -----              	  
     | [0,w]/     \		     | [0,w]/     \             	  
     -------|  5  |		     -------| 10  |             	  
       x:=0 \     /		       x:=0 \     /             	  
	     -----		             -----              	  

     NOTE: "w" represents infinity
*)


(* Use the semantics of DL *)
(* use "dcdiag.sml"; *)

(* Processes *)
fun process j i d D = 
    let val js = makestring(j:int) in
	[((i+1),(i+2),"xeq0","tt",0,~1),((i+2),(i+3),"tt","xeq"^js,0,D+1),
	 ((i+3),(i+4),"tt","tt",d,~1),((i+4),(i+1),"(~xeq"^js^")","tt",0,~1),
	 ((i+4),(i+5),"xeq"^js,"tt",0,~1),((i+5),(i+1),"tt","xeq0",0,~1)]
    end

fun process' j i d D = 
    let val js = makestring(j:int) in
	[((i+1),(i+2),"xeq0","tt",0,~1),((i+2),(i+3),"tt","xeq"^js,0,D),
	 ((i+3),(i+4),"tt","tt",d,~1),((i+4),(i+1),"(~xeq"^js^")","tt",0,~1),
	 ((i+4),(i+5),"xeq"^js,"tt",0,~1),((i+5),(i+5),"tt","tt",0,~1)]
    end

fun process'' j i d D = 
    let val js = makestring(j:int) in
	[((i+1),(i+2),"xeq11|xeq12","tt",0,~1),((i+2),(i+3),"tt","xeq"^js,0,D),
	 ((i+3),(i+4),"tt","tt",d,~1),((i+4),(i+1),"(~xeq"^js^")","tt",0,~1),
	 ((i+4),(i+5),"xeq"^js,"tt",0,~1),((i+5),(i+1),"tt","xeq1"^js,0,~1)]
    end


(* Restricted version *)
fun rprocess j i d D = 
    let val js = makestring(j:int) in
	[((i+1),(i+2),"xeq0","tt",0,~1),((i+2),(i+3),"tt","xeq"^js,0,D-1),
	 ((i+3),(i+4),"tt","tt",d,~1),((i+4),(i+5),"xeq"^js,"tt",0,~1),
	 ((i+5),(i+5),"tt","tt",0,~1)]
    end

(* The system specification *)
val d = 3
val D = 2
val system = ([("x",0,[0,1,2])],[1,6],[(process 1 0 d D),(process 2 5 d D)])

val system' = ([("x",0,[1,2])],[1,6],[(process' 1 0 d D),(process' 2 5 d D)])

val system'' = ([("x",11,[1,2,11,12])],[1,6],[(process'' 1 0 d D),(process'' 2 5 d D)])

val rsystem = ([("x",0,[0,1,2])],[1,6],[(rprocess 1 0 d D),(rprocess 2 5 d D)])


(*** Requirement Spec ***)

val mutex = "(~(<>('atl5'&'atl10')))"

(*** Proof ***)

val rfischer = "("^(diag rsystem)^" => "^mutex^")"
val fischer = "("^(diag system)^" => "^mutex^")"

val fischer' = "("^(diag system')^" => "^mutex^")"

val fischer'' = "("^(diag system'')^" => "^mutex^")"

(* jus routine *)
fun pcp st = implode (map (fn x => if x=";" then ";\n" else x) (explode st))


(* Tests *)

val f = frames [[((2),(3),"tt","xeq1",0,D),((4),(5),"tt","xeq1",0,D)]][("x",0,[1])]

val f' = frames' [[((2),(3),"tt","xeq1",0,D),((4),(5),"tt","xeq1",0,D)]][("x",0,[0])]

val f'' = frames'' [[((2),(3),"tt","xeq1",0,D),((4),(5),"tt","xeq1",0,D)]][("x",0,[0])]

val t1 = "[]((('xeq1';true)&('atl6'|'atl7'|'atl8'|'atl9'|'atl10')) => 'xeq1')"

val t1' = "([]((('xeq1';true)&'atl6') => 'xeq1')) &([]((('xeq1';true)&'atl7') => 'xeq1')) &([]((('xeq1';true)&'atl8') => 'xeq1')) &([]((('xeq1';true)&'atl9') => 'xeq1')) &([]((('xeq1';true)&'atl10') => 'xeq1'))& ";

val t1'' = "([]((('xeq1';true)&('atl6';'atl7')) => 'xeq1'))& ([]((('xeq1';true)&('atl8';'atl9')) => 'xeq1'))& ([]((('xeq1';true)&('atl9';'atl10')) => 'xeq1'))"

val t1 = t1'^t1''

val t2' = "([]((('xeq2';true)&'atl1') => 'xeq2')) &([]((('xeq2';true)&'atl2') => 'xeq2')) &([]((('xeq2';true)&'atl3') => 'xeq2')) &([]((('xeq2';true)&'atl4') => 'xeq2')) &([]((('xeq2';true)&'atl5') => 'xeq2'))& ";

val t'' = "([]((('xeq2';true)&('atl6';'atl7')) => 'xeq2'))& ([]((('xeq2';true)&('atl8';'atl9')) => 'xeq2'))& ([]((('xeq2';true)&('atl9';'atl10')) => 'xeq2'))"

val t1 = t1'^t1''

fun time3 a = conj (map (fn (i,j,c,e,L,U) => conj [(*(upper i c U),*)(lower c e L i j)]) a )
fun diag3 (vs,is,al) = conj (map (time3) al)
val fischer3 = "("^(diag3 ([("x",0,[0,1,2])],[3],[[((1),(2),"xeq0","tt",0,~1),((2),(3),"tt","xeq1",0,D),
	 ((3),(4),"tt","tt",d+2,~1),((4),(1),"(~xeq1"^")","tt",0,~1),
	 ((4),(5),"xeq1","tt",0,~1),((5),(1),"tt","xeq0",0,~1)]]))^" => "^mutex^")"
val fischer4 = "("^(diag3 ([("x",0,[0,1,2])],[3],[[((1),(2),"tt","tt",0,~1)]]))^" => "^mutex^")"


