(***************************************************************************
 File "printer.sml" Updated 1993-02-09
 Duration Calculus checker, version 2
 General printing functions
 Sestoft and Skakkebaek, Computer Science, Technical University of Denmark
***************************************************************************)

signature PRINTER = 
sig
    val prtbool  : bool -> string
    val prtint   : int -> string
    val prtreal  : real -> string
    val prtstring: string -> string
	
    val prt0tup  : unit -> string
    val prt1tup  : ('a -> string) -> ('a -> string)
    val prt2tup  : ('a -> string) * ('b -> string) -> ('a * 'b -> string)
    val prt3tup  : ('a -> string) * ('b -> string) * ('c -> string) 
                   -> ('a * 'b * 'c -> string)

    val prtseq   : ('a -> string) * string -> ('a list -> string) 
    val prtlist  : ('a -> string) -> ('a list -> string)
end

structure Prt : PRINTER =
struct
    (* Convert a base type value to a string *)
    fun prtbool b = makestring (b: bool)
    fun prtint  i = makestring (i: int)
    fun prtreal r = makestring (r: real)
    fun prtstring s = "\"" ^ s ^ "\""
	
    (* Convert a tuple to a string.  Each tuple converter is parametrized
     with print functions for its components *)
    fun prtpar s = "(" ^ s ^ ")"
    fun prt0tup () = prtpar ""
    fun prt1tup pr1 v1 = prtpar (pr1 v1)
    fun prt2tup (pr1, pr2) (v1, v2) = prtpar(pr1 v1 ^ "," ^ pr2 v2)
    fun prt3tup (pr1, pr2, pr3) (v1, v2, v3) = 
	prtpar(pr1 v1 ^ "," ^ pr2 v2 ^ "," ^ pr3 v3)

    (* Construct a sequence of elements, printed with pr, separated by sep *)
    fun prtseq (pr, sep) []      = ""
      | prtseq (pr, sep) [x]     = pr x
      | prtseq (pr, sep) (x::xs) = pr x ^ sep ^ prtseq(pr, sep) xs

    (* Convert a list to a string, converting the list elements with pr *)
    fun prtlist pr xs = "[" ^ prtseq (pr, ",") xs ^ "]"
end

(* Write string s on the file whose name is filename *)

fun writefile (filename: string, s: string): unit =
    let val outfile = open_out filename
	val _       = output(outfile, s)
	val _       = close_out outfile
    in () end

