(******************************************************************************
 THE FULL FISCHER ALGORITHM SPECIFIED IN THE DIAGRAM LANGUAGE (DL)
                     SIMPLIFIED VERSION
 92-12-06 Jens Ulrik Skakkebaek, Dept. of Computer Science,
          Technical University of Denmark
  PS version
******************************************************************************)
(*
     local   x: 0..2 where x=0
           d,D: integer     

         o                         o
         |                         |                               
         |                         |                               
         V                         V                               
       -----		       -----              	  
      /     \		      /     \             	  
      |  1  |		      |  6  |             	  
      \     /		      \     /             	  
       -----		       -----              	  
         |		         |                	  
         |		         |                	  
         | x=0?		         | x=0?           	  
         |		         |                	  
         V		         V                	  
       -----		       -----              	  
      /     \		      /     \             	  
      |  2  |		      |  7  |             	  
      \     /		      \     /             	  
       -----		       -----              	  
         |		         |                	  
         |		         |                	  
 [1,D+1] | x:=1		 [1,D+1] | x:=2           	  
         |		         |                	  
         V	    ||	         V                	  
       -----	    ||	       -----              	  
      /     \	    ||	      /     \             	  
      |  3  |		      |  8  |             	  
      \     /		      \     /             	  
       -----		       -----              	  
         |		         |                	  
         |		         |                	  
   [d,w] |		   [d,w] |                	  
         |		         |                	  
         V		         V                	  
       -----		       -----              	  
      /     \		      /     \             	  
      |  4  |		      |  9  |             	  
      \     /		      \     /             	  
       -----		       -----              	  
         |		         |                	  
         |		         |                	  
         | x=1?		         | x=2?           	  
         |		         |                	  
         V		         V                	  
       -----		       -----              	  
      /     \		      /     \             	  
      |  5  |		      | 10  |             	  
      \     /		      \     /             	  
       -----		       -----              	  
                             
     NOTE: "w" represents infinity
*)

(* Use the semantics of DL *)
(* use "rdcdiag.sml"; *)

(* Processes *)
fun process j i d D = 
    let val js = makestring(j:int) in
	[((i+1),(i+2),"xeq0","tt",1,~1),((i+2),(i+3),"tt","xeq"^js,1,D+1),
	 ((i+3),(i+4),"tt","tt",d,~1),((i+4),(i+5),"xeq"^js,"tt",1,~1),
	 ((i+5),(i+5),"tt","tt",1,~1)]
    end

(* The system specification *)
val d = 3
val D = 2

val system = ([("x",0,[0,1,2])],[1,6],[(process 1 0 d D),(process 2 5 d D)])

(*** Requirement Spec ***)

val mutex = "(~(<>('atl5'&'atl10')))"

(*** Proof ***)

val fischer = "("^(diag system)^" => "^mutex^")"
val fischer' = "("^(diag' system)^" => "^mutex^")"

(* jus routine *)
fun pcp st = implode (map (fn x => if x=";" then ";\n" else x) (explode st))


