(***************************************************************************
 File "valid.sml" Updated 1993-02-10
 Duration Calculus checker, version 2
 Checking a formula, or print the corresponding automaton
 Sestoft and Skakkebaek, Computer Science, Technical University of Denmark
***************************************************************************)


signature VALID =
sig
    (* SH 1997-07-24 Parse_error and Scan_error added to result to 
       provide error information *)

    datatype result  = 
	Valid
      | Unsatisfiable
      | Counter_example of string list
      | Parse_error of string
      | Scan_error of string
    
    val discrete_valid: string -> result
    val dense_valid: string -> result
    val prtdenseNFA: string -> unit
    val prtdenseDFA: string -> unit
    val prtdiscreteNFA: string -> unit
    val prtdiscreteDFA: string -> unit
    val timeit: ('a -> 'b) -> 'a -> 'b * string
end

functor Valid(structure Basic: BASIC 
	      and Formula: FORMULA
	      and Natsetset: NATSETSET
	      and Nfa: NFA
	      and Autoprint: AUTOPRINT
	      sharing type Basic.option = Nfa.option 
	      sharing type Natsetset.natset = Nfa.natset = Autoprint.natset
	      sharing type Nfa.natfun = Autoprint.natfun
	      sharing type Nfa.Bdd = Autoprint.Bdd
	      sharing type Formula.Formula = Nfa.Formula): VALID =
struct
    open Basic Natsetset Autoprint

    datatype result  = 
	Valid
      | Unsatisfiable
      | Counter_example of string list
      | Parse_error of string
      | Scan_error of string

    fun formsnames f = 
	let val formula = Formula.simplify (Formula.parse f)
	in (formula, Formula.statenames formula) end

    fun valid DFAofneg f = 
	let val (formula, snames) = formsnames f
	    val (tra, ini, fin, n) = DFAofneg (formula, snames)
	in if fin = empty              then Valid
	   else if member(ini, fin)    then Counter_example []
	   else if cardinality fin = n then Unsatisfiable
	   else case Nfa.accepting(tra, ini, fin, snames) of
	            None      => Valid
		  | Some path => Counter_example path
	end

    fun dense_valid f = 
      valid Nfa.denseDFAofneg f
      handle Formula.Scanerror s => Scan_error s
           | Formula.Parseerror s => Parse_error s
      
    fun discrete_valid f = 
      valid Nfa.discreteDFAofneg f
      handle Formula.Scanerror s => Scan_error s
           | Formula.Parseerror s => Parse_error s

    fun prtdenseNFA f    =
	let val (formula, snames) = formsnames f
	in print (nfaprint (Nfa.denseNFA (formula, snames), snames)) end

    fun prtdiscreteNFA f = 
	let val (formula, snames) = formsnames f
	in print (nfaprint (Nfa.discreteNFA (formula, snames), snames)) end

    fun prtdenseDFA f    = 
	let val (formula, snames) = formsnames f
	in print (dfaprint (Nfa.NFA2DFA (Nfa.denseNFA (formula, snames), 0), 
			    snames)) 
	end

    fun prtdiscreteDFA f = 
	let val (formula, snames) = formsnames f
	in print (dfaprint (Nfa.NFA2DFA (Nfa.discreteNFA (formula, snames), 0),
			    snames))
	end
end
